package sun.java2d.loops;

import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import sun.security.action.GetPropertyAction;
import java.lang.reflect.Field;
import sun.java2d.pipe.Region;
import java.awt.Rectangle;
import java.awt.AlphaComposite;
import sun.awt.image.BufImgSurfaceData;
import java.awt.image.BufferedImage;
import sun.java2d.SurfaceData;
import java.security.AccessController;
import java.io.FileNotFoundException;
import java.security.PrivilegedAction;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.HashMap;

public abstract class GraphicsPrimitive
{
    private String methodSignature;
    private int uniqueID;
    private static int unusedPrimID;
    private SurfaceType sourceType;
    private CompositeType compositeType;
    private SurfaceType destType;
    private long pNativePrim;
    static HashMap traceMap;
    public static int traceflags;
    public static String tracefile;
    public static PrintStream traceout;
    public static final int TRACELOG = 1;
    public static final int TRACETIMESTAMP = 2;
    public static final int TRACECOUNTS = 4;
    private String cachedname;
    
    public static final synchronized int makePrimTypeID() {
        if (GraphicsPrimitive.unusedPrimID > 255) {
            throw new InternalError("primitive id overflow");
        }
        return GraphicsPrimitive.unusedPrimID++;
    }
    
    public static final synchronized int makeUniqueID(int n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return n << 24 | surfaceType2.getUniqueID() << 16 | compositeType.getUniqueID() << 8 | surfaceType.getUniqueID();
    }
    
    protected GraphicsPrimitive(String methodSignature, int n, SurfaceType sourceType, CompositeType compositeType, SurfaceType destType) {
        this.methodSignature = methodSignature;
        this.sourceType = sourceType;
        this.compositeType = compositeType;
        this.destType = destType;
        if (sourceType == null || compositeType == null || destType == null) {
            this.uniqueID = n << 24;
        }
        else {
            this.uniqueID = makeUniqueID(n, sourceType, compositeType, destType);
        }
    }
    
    protected GraphicsPrimitive(long pNativePrim, String methodSignature, int n, SurfaceType sourceType, CompositeType compositeType, SurfaceType destType) {
        this.pNativePrim = pNativePrim;
        this.methodSignature = methodSignature;
        this.sourceType = sourceType;
        this.compositeType = compositeType;
        this.destType = destType;
        if (sourceType == null || compositeType == null || destType == null) {
            this.uniqueID = n << 24;
        }
        else {
            this.uniqueID = makeUniqueID(n, sourceType, compositeType, destType);
        }
    }
    
    public final int getUniqueID() {
        return this.uniqueID;
    }
    
    public final String getSignature() {
        return this.methodSignature;
    }
    
    public final int getPrimTypeID() {
        return this.uniqueID >>> 24;
    }
    
    public final long getNativePrim() {
        return this.pNativePrim;
    }
    
    public final SurfaceType getSourceType() {
        return this.sourceType;
    }
    
    public final CompositeType getCompositeType() {
        return this.compositeType;
    }
    
    public final SurfaceType getDestType() {
        return this.destType;
    }
    
    public final boolean satisfies(String s, SurfaceType superType, CompositeType superType2, SurfaceType superType3) {
        if (s != this.methodSignature) {
            return false;
        }
        while (superType != null) {
            if (superType.equals(this.sourceType)) {
                while (superType2 != null) {
                    if (superType2.equals(this.compositeType)) {
                        while (superType3 != null) {
                            if (superType3.equals(this.destType)) {
                                return true;
                            }
                            superType3 = superType3.getSuperType();
                        }
                        return false;
                    }
                    superType2 = superType2.getSuperType();
                }
                return false;
            }
            superType = superType.getSuperType();
        }
        return false;
    }
    
    final boolean satisfiesSameAs(GraphicsPrimitive graphicsPrimitive) {
        return this.methodSignature == graphicsPrimitive.methodSignature && this.sourceType.equals(graphicsPrimitive.sourceType) && this.compositeType.equals(graphicsPrimitive.compositeType) && this.destType.equals(graphicsPrimitive.destType);
    }
    
    public abstract GraphicsPrimitive makePrimitive(SurfaceType p0, CompositeType p1, SurfaceType p2);
    
    public abstract GraphicsPrimitive traceWrap();
    
    public static boolean tracingEnabled() {
        return GraphicsPrimitive.traceflags != 0;
    }
    
    private static PrintStream getTraceOutputFile() {
        if (GraphicsPrimitive.traceout == null) {
            if (GraphicsPrimitive.tracefile != null) {
                FileOutputStream out = (FileOutputStream)AccessController.doPrivileged(new PrivilegedAction<FileOutputStream>() {
                    @Override
                    public FileOutputStream run() {
                        try {
                            return new FileOutputStream(GraphicsPrimitive.tracefile);
                        }
                        catch (FileNotFoundException ex) {
                            return null;
                        }
                    }
                });
                if (out != null) {
                    GraphicsPrimitive.traceout = new PrintStream(out);
                }
                else {
                    GraphicsPrimitive.traceout = System.err;
                }
            }
            else {
                GraphicsPrimitive.traceout = System.err;
            }
        }
        return GraphicsPrimitive.traceout;
    }
    
    public static synchronized void tracePrimitive(Object x) {
        if ((GraphicsPrimitive.traceflags & 0x4) != 0x0) {
            if (GraphicsPrimitive.traceMap == null) {
                GraphicsPrimitive.traceMap = new HashMap();
                GraphicsPrimitive.TraceReporter.setShutdownHook();
            }
            Object value = GraphicsPrimitive.traceMap.get(x);
            if (value == null) {
                value = new int[] { 0 };
                GraphicsPrimitive.traceMap.put(x, value);
            }
            int[] array = (int[])(int[])value;
            int n = 0;
            ++array[n];
        }
        if ((GraphicsPrimitive.traceflags & 0x1) != 0x0) {
            PrintStream traceOutputFile = getTraceOutputFile();
            if ((GraphicsPrimitive.traceflags & 0x2) != 0x0) {
                traceOutputFile.print(System.currentTimeMillis() + ": ");
            }
            traceOutputFile.println(x);
        }
    }
    
    protected void setupGeneralBinaryOp(GraphicsPrimitive.GeneralBinaryOp generalBinaryOp) {
        int primTypeID = generalBinaryOp.getPrimTypeID();
        String signature = generalBinaryOp.getSignature();
        SurfaceType sourceType = generalBinaryOp.getSourceType();
        CompositeType compositeType = generalBinaryOp.getCompositeType();
        SurfaceType destType = generalBinaryOp.getDestType();
        Blit converter = createConverter(sourceType, SurfaceType.IntArgb);
        GraphicsPrimitive graphicsPrimitive = GraphicsPrimitiveMgr.locatePrim(primTypeID, SurfaceType.IntArgb, compositeType, destType);
        Blit converter2;
        Blit converter3;
        if (graphicsPrimitive != null) {
            converter2 = null;
            converter3 = null;
        }
        else {
            graphicsPrimitive = getGeneralOp(primTypeID, compositeType);
            if (graphicsPrimitive == null) {
                throw new InternalError("Cannot construct general op for " + signature + " " + compositeType);
            }
            converter2 = createConverter(destType, SurfaceType.IntArgb);
            converter3 = createConverter(SurfaceType.IntArgb, destType);
        }
        generalBinaryOp.setPrimitives(converter, converter2, graphicsPrimitive, converter3);
    }
    
    protected void setupGeneralUnaryOp(GraphicsPrimitive.GeneralUnaryOp generalUnaryOp) {
        int primTypeID = generalUnaryOp.getPrimTypeID();
        generalUnaryOp.getSignature();
        CompositeType compositeType = generalUnaryOp.getCompositeType();
        SurfaceType destType = generalUnaryOp.getDestType();
        Blit converter = createConverter(destType, SurfaceType.IntArgb);
        GraphicsPrimitive generalOp = getGeneralOp(primTypeID, compositeType);
        Blit converter2 = createConverter(SurfaceType.IntArgb, destType);
        if (converter == null || generalOp == null || converter2 == null) {
            throw new InternalError("Cannot construct binary op for " + compositeType + " " + destType);
        }
        generalUnaryOp.setPrimitives(converter, generalOp, converter2);
        return;
    }
    
    protected static Blit createConverter(SurfaceType obj, SurfaceType obj2) {
        if (obj.equals(obj2)) {
            return null;
        }
        Blit fromCache = Blit.getFromCache(obj, CompositeType.SrcNoEa, obj2);
        if (fromCache == null) {
            throw new InternalError("Cannot construct converter for " + obj + "=>" + obj2);
        }
        return fromCache;
    }
    
    protected static SurfaceData convertFrom(Blit blit, SurfaceData surfaceData, int n, int n2, int n3, int n4, SurfaceData surfaceData2) {
        return convertFrom(blit, surfaceData, n, n2, n3, n4, surfaceData2, 2);
    }
    
    protected static SurfaceData convertFrom(Blit blit, SurfaceData surfaceData, int n, int n2, int width, int height, SurfaceData data, int imageType) {
        if (data != null) {
            Rectangle bounds = data.getBounds();
            if (width > bounds.width || height > bounds.height) {
                data = null;
            }
        }
        if (data == null) {
            data = BufImgSurfaceData.createData(new BufferedImage(width, height, imageType));
        }
        blit.Blit(surfaceData, data, AlphaComposite.Src, null, n, n2, 0, 0, width, height);
        return data;
    }
    
    protected static void convertTo(Blit blit, SurfaceData surfaceData, SurfaceData surfaceData2, Region region, int n, int n2, int n3, int n4) {
        if (blit != null) {
            blit.Blit(surfaceData, surfaceData2, AlphaComposite.Src, region, 0, 0, n, n2, n3, n4);
        }
    }
    
    protected static GraphicsPrimitive getGeneralOp(int n, CompositeType compositeType) {
        return GraphicsPrimitiveMgr.locatePrim(n, SurfaceType.IntArgb, compositeType, SurfaceType.IntArgb);
    }
    
    public static String simplename(Field[] array, Object o) {
        for (int i = 0; i < array.length; ++i) {
            Field field = array[i];
            try {
                if (o == field.get(null)) {
                    return field.getName();
                }
            }
            catch (Exception ex) {}
        }
        return "\"" + o.toString() + "\"";
    }
    
    public static String simplename(SurfaceType surfaceType) {
        return simplename(SurfaceType.class.getDeclaredFields(), surfaceType);
    }
    
    public static String simplename(CompositeType compositeType) {
        return simplename(CompositeType.class.getDeclaredFields(), compositeType);
    }
    
    @Override
    public String toString() {
        if (this.cachedname == null) {
            String str = this.methodSignature;
            int index = str.indexOf(40);
            if (index >= 0) {
                str = str.substring(0, index);
            }
            this.cachedname = this.getClass().getName() + "::" + str + "(" + simplename(this.sourceType) + ", " + simplename(this.compositeType) + ", " + simplename(this.destType) + ")";
        }
        return this.cachedname;
    }
    
    static PrintStream access$000() {
        return getTraceOutputFile();
    }
    
    static {
        GraphicsPrimitive.unusedPrimID = 1;
        String str = (String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.trace"));
        if (str != null) {
            boolean n = false;
            int traceflags = 0;
            StringTokenizer stringTokenizer = new StringTokenizer(str, ",");
            while (stringTokenizer.hasMoreTokens()) {
                String nextToken = stringTokenizer.nextToken();
                if (nextToken.equalsIgnoreCase("count")) {
                    traceflags |= 0x4;
                }
                else if (nextToken.equalsIgnoreCase("log")) {
                    traceflags |= 0x1;
                }
                else if (nextToken.equalsIgnoreCase("timestamp")) {
                    traceflags |= 0x2;
                }
                else if (nextToken.equalsIgnoreCase("verbose")) {
                    n = true;
                }
                else if (nextToken.regionMatches(true, 0, "out:", 0, 4)) {
                    GraphicsPrimitive.tracefile = nextToken.substring(4);
                }
                else {
                    if (!nextToken.equalsIgnoreCase("help")) {
                        System.err.println("unrecognized token: " + nextToken);
                    }
                    System.err.println("usage: -Dsun.java2d.trace=[log[,timestamp]],[count],[out:<filename>],[help],[verbose]");
                }
            }
            if (!(n == 0)) {
                System.err.print("GraphicsPrimitive logging ");
                if ((traceflags & 0x1) != 0x0) {
                    System.err.println("enabled");
                    System.err.print("GraphicsPrimitive timetamps ");
                    if ((traceflags & 0x2) != 0x0) {
                        System.err.println("enabled");
                    }
                    else {
                        System.err.println("disabled");
                    }
                }
                else {
                    System.err.println("[and timestamps] disabled");
                }
                System.err.print("GraphicsPrimitive invocation counts ");
                if ((traceflags & 0x4) != 0x0) {
                    System.err.println("enabled");
                }
                else {
                    System.err.println("disabled");
                }
                System.err.print("GraphicsPrimitive trace output to ");
                if (GraphicsPrimitive.tracefile == null) {
                    System.err.println("System.err");
                }
                else {
                    System.err.println("file '" + GraphicsPrimitive.tracefile + "'");
                }
            }
            GraphicsPrimitive.traceflags = traceflags;
        }
    }
    
    public static class TraceReporter extends Thread
    {
        public static void setShutdownHook() {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    GraphicsPrimitive.TraceReporter hook = new GraphicsPrimitive.TraceReporter();
                    hook.setContextClassLoader(null);
                    Runtime.getRuntime().addShutdownHook(hook);
                    return null;
                }
            });
        }
        
        @Override
        public void run() {
            PrintStream access$000 = GraphicsPrimitive.access$000();
            Iterator<Object> iterator;
            long lng;
            int i;
            Map.Entry<Object, Object> entry;
            Object key;
            int[] array;
            for (iterator = GraphicsPrimitive.traceMap.entrySet().iterator(), lng = 0L, i = 0; iterator.hasNext(); ++i, lng += (long)array[0]) {
                entry = (Map.Entry)iterator.next();
                key = entry.getKey();
                array = (int[])(int[])entry.getValue();
                if (array[0] == 1) {
                    access$000.print("1 call to ");
                }
                else {
                    access$000.print(array[0] + " calls to ");
                }
                access$000.println(key);
            }
            if (i == 0) {
                access$000.println("No graphics primitives executed");
            }
            else if (i > 1) {
                access$000.println(lng + " total calls to " + i + " different primitives");
            }
        }
    }
    
    protected interface GeneralBinaryOp
    {
        public abstract void setPrimitives(Blit p0, Blit p1, GraphicsPrimitive p2, Blit p3);
        
        public abstract SurfaceType getSourceType();
        
        public abstract CompositeType getCompositeType();
        
        public abstract SurfaceType getDestType();
        
        public abstract String getSignature();
        
        public abstract int getPrimTypeID();
    }
    
    protected interface GeneralUnaryOp
    {
        public abstract void setPrimitives(Blit p0, GraphicsPrimitive p1, Blit p2);
        
        public abstract CompositeType getCompositeType();
        
        public abstract SurfaceType getDestType();
        
        public abstract String getSignature();
        
        public abstract int getPrimTypeID();
    }
}
